###########################R syntax file to estimate standard two-level meta-analytic models################################################

# Syntax accompanies Multivariate Behavioral Research article titled: 
# "On the multilevel nature of meta-analysis: A tutorial, comparison of software programs, and discussion of analytic choices"
# by Dena A. Pastor and Rory A. Lazowski.

# Please send any questions to Dena Pastor: pastorda@jmu.edu.

#Data set in this example is from Raudenbush (1984)
#The id variable is "study", the effect size is "yi", the within-study variance is "vi", and the moderator used in the example is "weeks".

#Eight models are fit to the data: 
	#Fixed Effects model, no moderators
 	#Fixed Effects model, with moderators
	#Random Effects model, no moderators, DL estimation
	#Random Effects model, no moderators, REML estimation
	#Random Effects model, no moderators, ML estimation
	#Random Effects model, with moderators, DL estimation
	#Random Effects model, with moderators, REML estimation
	#Random Effects model, with moderators, ML estimation


#The following webpage by Wolfgang Viecthbauer illustrates how to analyze the same
#data using the metafor package in R: 
#http://www.metafor-project.org/doku.php/analyses:raudenbush2009

#Much of the code below was taken directly from this website

##########################################################################################
#load metafor package 
library(metafor)

#If you don't have the package already loaded, select Packages from drop-down menu,
#Install packges, select a geographically nearby mirror site, and select metafor from list of 
#available packages

#load data for tutorial - it comes with metafor package
#calling data set "dat"
dat<-get(data(dat.raudenbush1985))

#view data
dat

########################Fixed Effects Model, No Moderators################################

FE_NOMOD<-rma(yi,vi,data=dat,digits=7, method='FE')

#view results
FE_NOMOD

#create forest plot of results
forest(FE_NOMOD)

########################Fixed Effects Model, Moderators##################################

FE_MOD<-rma(yi,vi,data=dat,digits=7 , mods=~weeks, method="FE")

#view results
FE_MOD

#####################Random Effects Model, No Moderators################################

#When method="DL" is specified, a RE model with the DerSimonian and Lairds estimator (DL) is used

RE_NOMOD_DL<-rma(yi,vi,data=dat,digits=4, method="DL")

#view results
RE_NOMOD_DL

#obtain CIs for tau^2, tau, I^2, H^2
confint(RE_NOMOD_DL)

#obtain credibility/prediction interval
predict(RE_NOMOD_DL)

#create forest plot of results
forest(RE_NOMOD_DL)

######

#When method="REML" is specified, a RE model with restricted maximum likelihood (REML) is used

RE_NOMOD_REML<-rma(yi,vi,data=dat,digits=4, method="REML")
RE_NOMOD_REML

#obtain CIs for tau^2, tau, I^2, H^2
confint(RE_NOMOD_REML)

#obtain credibility/prediction interval
predict(RE_NOMOD_REML)

#create forest plot of results
forest(RE_NOMOD_REML)

######

#When method="ML" is specified, a RE model with full maximum likelihood (ML) is used

RE_NOMOD_ML<-rma(yi,vi,data=dat,digits=4, method="ML")
RE_NOMOD_ML 

#obtain CIs for tau^2, tau, I^2, H^2
confint(RE_NOMOD_ML)

#obtain credibility/prediction interval
predict(RE_NOMOD_ML)

#create forest plot of results
forest(RE_NOMOD_ML)

#######################Random Effects Model, Moderators#######################

#DerSimonian and Lairds estimator (DL)

RE_MOD_DL<-rma(yi,vi,data=dat,digits=4,mods=~weeks, method="DL")
RE_MOD_DL

#obtain CIs for tau^2, tau, I^2, H^2
confint(RE_MOD_DL)

######

#restricted maximum likelihood (REML) 

RE_MOD_REML<-rma(yi,vi,data=dat,digits=4,mods=~weeks, method="REML")
RE_MOD_REML

#obtain CIs for tau^2, tau, I^2, H^2
confint(RE_MOD_REML)

######

#full maximum likelihood

RE_MOD_ML<-rma(yi,vi,data=dat,digits=4,mods=~weeks, method="ML")
RE_MOD_ML

#obtain CIs for tau^2, tau, I^2, H^2
confint(RE_MOD_ML)

#####################################################################################
