/*************************SAS syntax file to estimate standard two-level meta-analytic models****************************************

Syntax accompanies Multivariate Behavioral Research article titled: 
"On the multilevel nature of meta-analysis: A tutorial, comparison of software programs, and discussion of analytic choices"
by Dena A. Pastor and Rory A. Lazowski.

Please send any questions to Dena Pastor: pastorda@jmu.edu.

Data set in this example is from Raudenbush (1984)
The id variable is "study", the effect size is "yi", the within-study variance is "vi", and the moderator used in the example is "weeks".

Six models are fit to the data using PROC MIXED: 
	Fixed Effects model, no moderators
 	Fixed Effects model, with moderators
	Random Effects model, no moderators, REML estimation
	Random Effects model, no moderators, ML estimation
	Random Effects model, with moderators, REML estimation
	Random Effects model, with moderators, ML estimation

Note: DerSimonian and Lairds (DL; 1986) estimator is not available for Random Effects meta-analytic models in PROC MIXED. 

*****************************************************************************************************************************************/
*To the left of the equal sign below put the path where output files will be saved;
%let outpath=C:\Users\Desktop\output;

*Data;
data d1;
input study yi vi weeks;
datalines;
1	0.03	0.0156	2
2	0.12	0.0216	21
3	-0.14	0.0279	19
4	1.18	0.1391	0
5	0.26	0.1362	0
6	-0.06	0.0106	3
7	-0.02	0.0106	17
8	-0.32	0.0484	24
9	0.27	0.0269	0
10	0.80	0.0630	1
11	0.54	0.0912	0
12	0.18	0.0497	0
13	-0.02	0.0835	1
14	0.23	0.0841	2
15	-0.18	0.0253	17
16	-0.06	0.0279	5
17	0.30	0.0193	1
18	0.07	0.0088	2
19	-0.07	0.0303	7
;
run;


/*******************************************************************************************

First, we create two data sets:

1) forFE - contains known values (v) of within-study variances; used in proc mixed for FE models.
2) forRE - contains starting value for between-study variance (tau) and known values (v) of within-study variances; 
   used in proc mixed for RE models.

This step is not necessary, but allows a data set with the known within-study variances and starting values for between-study
variances to be used on the PARMS subcommand of PROC MIXED. An alternative approach involves typing the values directly into hte
PARMS subcommand as shown in:

Sheu, C.-F., & Suzuki, S. (2001). Meta-analysis using linear mixed models. Behavior Research Methods, Instruments, & Computers, 33(2), 102-107.

Konstantopoulos, S. (2011). Fixed effects and variance components estimation in three-level meta-analysis. Research Synthesis Methods, 2,
6176. 
**********************************************************************************************************************************/

proc transpose data=d1 out=d2; var vi; run;

%let count=19;					*enter number of effect sizes here*;
%let count2=%eval(&count+1);	*this computes one plus number of effect sizes (add one for between study variance);

*creating forFE;
data forFE(keep=covp1-covp&count); set d2;
array col(*) col1-col&count;
array covp(*) covp1-covp&count;	
do i=1 to &count;
  covp(i)=col(i);
end;
run;

*creating forRE;
data forRE(keep=covp1-covp&count2); set d2;
array col(*) col1-col&count;
array covp(*) covp1-covp&count2;	
do i=1 to &count2;
 if i=1 then covp(i)=0.01; *starting value of 0.01 for tau;
 else covp(i)=col(i-1);
end;
run;

/******************************************************************************************************/

***************************Fixed Effects Model, No Moderators******************************************;
ods rtf file="&outpath\FE_NOMOD.rtf";
proc mixed covtest data=d1;
 class study; 	
 model yi =   / solution ddfm=bw ;
 repeated/ group=study;
 parms /EQCONS=1 to &count parmsdata=forFE;
 title1 'Fixed Effects Model, No Moderators: FE_NOMOD';
run;
ods rtf close;

********************************Fixed Effects Model, Moderators*******************************;
ods rtf file="&outpath\FE_MOD.rtf";
proc mixed covtest data=d1;
 class study; 	
 model yi =  weeks / solution ddfm=bw ;
 repeated/ group=study;
 parms /EQCONS=1 to &count parmsdata=forFE;
 title1 'Fixed Effects Model, Moderators:FE_MOD';
run;
ods rtf close;

***************************Random Effects Model, No Moderators***********************************;
*When method="REML" is specified, a RE model with restricted maximum likelihood (REML) is used;
ods rtf file="&outpath\RE_NOMOD_REML.rtf";
proc mixed covtest data=d1 method=REML scoring=1000;
 class study; 	
 model yi =   / solution ddfm=bw ;
 random int/sub=study  ;
 repeated/ group=study;
 parms /EQCONS=2 to &count2 parmsdata=forRE;
 title1 'Random Effects Model, No Moderators, REML: RE_NOMOD_REML';
run;
ods rtf close;

*When method="ML" is specified, a RE model with full maximum likelihood (ML) is used;
ods rtf file="&outpath\RE_NOMOD_ML.rtf";
proc mixed covtest data=d1 method=ML scoring=1000;
 class study; 	
 model yi =   / solution ddfm=bw ;
 random int/sub=study  ;
 repeated/ group=study;
 parms /EQCONS=2 to &count2 parmsdata=forRE;
 title1 'Random Effects Model, No Moderators, ML: RE_NOMOD_ML';
run;
ods rtf close;
********************************Random Effects Model, Moderators*******************************;

*When method="REML" is specified, a RE model with restricted maximum likelihood (REML) is used;
ods rtf file="&outpath\RE_MOD_REML.rtf";
proc mixed covtest data=d1 method=REML scoring=1000;
 class study; 	
 model yi = weeks / solution ddfm=bw ;
 random int/sub=study  ;
 repeated/ group=study;
 parms /EQCONS=2 to &count2 parmsdata=forRE;
 title1 'Random Effects Model, Moderators, REML:RE_MOD_REML';
run;
ods rtf close;

*When method="ML" is specified, a RE model with full maximum likelihood (ML) is used;
ods rtf file="&outpath\RE_MOD_ML.rtf" ;
proc mixed covtest data=d1 method=ML scoring=1000;
 class study; 	
 model yi =  weeks / solution ddfm=bw ;
 random int/sub=study  ;
 repeated/ group=study;
 parms /EQCONS=2 to &count2 parmsdata=forRE;
 title1 'Random Effects Model, Moderators, ML: RE_MOD_ML';
run;
ods rtf close;

***Note about the use of scoring=1000 in proc mixed based on personal communication with Viecthbauer (2016):

Proc mixed can give slightly different SEs than metafor because proc mixed computes SEs based on the observed information matrix, 
while metafor uses the expected information matrix. The use of 'scoring=1000' forced proc mixed to stick to the Fisher scoring algorithm
and also to use the expected information matrix for computing SEs. When scoring=1000, the results of proc mixed and metafor match.

***;
